/*$
Copyright (c) 2015, Azel
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:
    * Redistributions of source code must retain the above copyright
      notice, this list of conditions and the following disclaimer.
    * Redistributions in binary form must reproduce the above copyright
      notice, this list of conditions and the following disclaimer in the
      documentation and/or other materials provided with the distribution.
    * Neither the name of the <organization> nor the
      names of its contributors may be used to endorse or promote products
      derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL <COPYRIGHT HOLDER> BE LIABLE FOR ANY
DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
$*/

/******************************
 * HTML 出力 文字列生成
 ******************************/

#include "mDef.h"
#include "mStr.h"

#include "zipfile.h"


//==========================
// sub
//==========================


/** HTML ヘッダ */

static void _set_html_header(mStr *str)
{
	mStrSetText(str,
"<!DOCTYPE html>\n"
"<html><head>\n"
"<meta http-equiv=\"Content-Type\" content=\"text/html; charset=UTF-8\">\n"
"<style type=\"text/css\">\n"
"body{ font-family:sans-serif; font-size:12pt; line-height:130%; margin:20px; }\n"
"a:link{ color:blue }\n"
"a:visited{ color:#009 }\n"
"a:hover{ color:#f0c }\n"
"h2{ margin:1em -5px; font-size:100%; background-color:#8B93A3; color:white; padding:3px 10px; line-height:100%; }\n"
"</style>\n"
"</head><body>\n");
}

/** HTML フッタ */

static void _append_html_footer(mStr *str)
{
	mStrAppendText(str, "\n</body></html>");
}

/** 特殊文字変換して追加 */

static void _append_htmltxt(mStr *str,const char *text)
{
	char c;
	const char *pc;
	
	for(; (c = *text); text++)
	{
		if(c < 0)
			pc = NULL;
		else if(c == ' ')
			pc = "&nbsp;";
		else if(c == '&')
			pc = "&amp;";
		else if(c == '<')
			pc = "&lt;";
		else if(c == '>')
			pc = "&gt;";
		else if(c == '"')
			pc = "&quot;";
		else
			pc = NULL;
		
		if(pc)
			mStrAppendText(str, pc);
		else
			mStrAppendChar(str, c);
	}
}


//==========================
// main
//==========================


/** ディレクトリのファイルリスト生成
 *
 * @param dirinfo      NULL でトップ
 * @param path_decode  ディレクトリパス (デコード済み)。NULL でトップ。 */

void html_directory_filelist(mStr *strdst,ZipFileInfo *dirinfo,const char *path_decode)
{
	ZipFileInfo info;
	mStr str = MSTR_INIT;
	int ret;

	_set_html_header(strdst);

	//ディレクトリのパス

	mStrAppendText(strdst, "<h2>/");

	if(path_decode)
		_append_htmltxt(strdst, path_decode);

	mStrAppendText(strdst, "</h2>\n");

	//先頭アイテム

	if(!dirinfo)
		ret = zip_move(&info, ZIP_MOVE_TOP);
	else
	{
		info = *dirinfo;

		//親へ
		mStrAppendText(strdst, "<a href=\"../\">../</a><br>\n");

		//

		ret = zip_move(&info, ZIP_MOVE_FIRST_CHILD);
	}
		
	//ファイルリスト

	if(ret)
	{
		do
		{
			//URL

			mStrSetURLEncode(&str, info.name);

			if(info.flags & ZIPFILEINFO_F_DIRECTORY)
				mStrAppendChar(&str, '/');

			mStrAppendFormat(strdst, "<a href=\"%t\">", &str);

			//表示用

			_append_htmltxt(strdst, info.name);
		
			if(info.flags & ZIPFILEINFO_F_DIRECTORY)
				mStrAppendChar(strdst, '/');
			
			mStrAppendText(strdst, "</a><br>\n");

		} while(zip_move(&info, ZIP_MOVE_NEXT));
	}

	mStrFree(&str);

	//
	
	_append_html_footer(strdst);
}
